//
//  ViewController.m
//  SDKTestApp
//
//  Created by Vidyalaxmi V Shenoy on 20/07/15.
//  Copyright (c) 2015 Webaroo. All rights reserved.
//

#import "HomeViewController.h"
#import <TeamchatSDK/TeamchatSDK.h>
#import "SettingsViewController.h"
#import "SVProgressHUD.h"
#import "LoginViewController.h"
#import "CustomContactsViewController.h"
#import "HomeTableViewCell.h"
#import "ListTableViewController.h"
#import "RegisterUsingVerifyCodeAPIsViewController.h"

typedef NS_OPTIONS(NSInteger, TextFieldType)
{
    TextFieldTypeDefault,
    TextFieldTypeEmail,
    TextFieldTypeUserID,
    TextFieldTypeToken,
    TextFieldTypeEnvironment
};

typedef NS_OPTIONS(NSInteger, TableViewRowType)
{
    TableViewRowTypeAddCustomContacts,
    TableViewRowTypeShowChatsList,
    TableViewRowTypeShowChatRoom,
    TableViewRowTypeProfile,
    TableViewRowTypeSettings,
    TableViewRowTypePublicGroups,
    TableViewRowTypeCreateNewGroup,
    TableViewRowTypeAddMembersToGroup,
    TableViewRowTypeGroupFilter,
    TableViewRowTypeUpdateUserProfile
};

@interface HomeViewController () <UIActionSheetDelegate, UIAlertViewDelegate, LoginViewControllerDelegate, TCCustomContactProvider,
                                    CustomContactsViewControllerDelegate, TeamchatJsIOSBridgeInjectorDelegate>
{
    NSString *selectedEnvironment;
}

@property (weak, nonatomic) IBOutlet UIBarButtonItem *settingsBarButtonItem;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *loginOrResetBarButtonItem;
@property (weak, nonatomic) IBOutlet UITableView *homeTableView;

@property (nonatomic, assign) LoginOptionType loginOptionType;
@property (nonatomic) NSArray *customContactsArray;

@end

@implementation HomeViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self.settingsBarButtonItem setImage:[[UIImage imageNamed:@"settings_icon"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]];
    
    self.homeTableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    
    [[TeamchatJsIOSBridgeInjector sharedInjectorInstance] setInjectorDelegate:self forScheme:@"TestJSLoadSchema"];
    
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    if ([Teamchat sessionState] == TCSessionStateActive || [Teamchat sessionState] == TCSessionStateLoggedIn)
    {
        [self changeViewsOnSuccessfulLogin];
    }
    if ([Teamchat showContactOptions] == 0)
    {
        [Teamchat setShowContactOptions:TCShowContactFromLocalAddressBook | TCShowContactFromServerAddressBook | TCShowContactFromCustomAddressBook];
    }
}

#pragma mark - Action methods

- (IBAction)loginOrResetButtonTapped:(id)sender
{
    if ([Teamchat sessionState] == TCSessionStateActive || [Teamchat sessionState] == TCSessionStateLoggedIn)
    {
        [[[UIAlertView alloc] initWithTitle:@"Confirm" message:@"Are you sure you want to reset Teamchat session?"
                                   delegate:self cancelButtonTitle:@"No" otherButtonTitles:@"Yes", nil] show];
    }
    else
    {
        [self showLoginOptions];
    }
}

- (void)showLoginOptions
{
    UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:@"Login options"
                                                             delegate:self cancelButtonTitle:@"Cancel" destructiveButtonTitle:nil
                                                    otherButtonTitles:@"Login by setting AuthToken, userID and EmailID", @"Teamchat login with prefilled email", @"Teamchat login",@"Using Register & Verify Code APIs", nil];
    [actionSheet showInView:self.view];
}

- (void)showLoginView
{
    UIViewController *loginViewController = [Teamchat teamchatLoginViewController:^(BOOL success, NSError *error, NSString *errMsg) {
        if (success)
        {
            [[[UIAlertView alloc] initWithTitle:@"" message:@"Teamchat login successful" delegate:nil
                              cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
            [self changeViewsOnSuccessfulLogin];
        }
        else
        {
            NSString *msg = error ? [error localizedDescription] : errMsg;
            
            if (msg)
            {
                [[[UIAlertView alloc] initWithTitle:@"Failed" message:msg delegate:nil
                                  cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
            }
        }
    }];
    if (loginViewController)
    {
        [self.navigationController pushViewController:loginViewController animated:YES];
    }
}

- (void)changeViewsOnSuccessfulLogin
{
    [self.loginOrResetBarButtonItem setTitle:@"Reset Session"];
    [self.homeTableView reloadData];
}

- (void)showGroupsList
{
    if ([Teamchat sessionState] > TCSessionStateInactive)
    {
        [self performSegueWithIdentifier:@"RoomsListSegue" sender:nil];
    }
}

- (void)showViewForRowType:(TableViewRowType)type
{
    __weak HomeViewController *weakself = self;

    if ([Teamchat sessionState] > TCSessionStateInactive)
    {
        if (type == TableViewRowTypeShowChatsList)
        {
            [Teamchat setCustomAddressBookContactProvider:self];
        }
        
        [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
        [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
            [SVProgressHUD dismiss];
            
            if (success)
            {
                UIViewController *viewController = [weakself viewControllerForRowType:type];
                
                if (viewController)
                {
                    if (type == TableViewRowTypePublicGroups)
                    {
                        [weakself.navigationController pushViewController:viewController animated:YES];
                    }
                    else
                    {
                        [weakself presentViewController:viewController animated:YES completion:nil];
                    }
                }
            }
            else
            {
                NSString *msg = error ? [error localizedDescription] : errMsg;
                
                if (msg)
                {
                    [[[UIAlertView alloc] initWithTitle:@"Failed" message:msg delegate:nil
                                      cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
                }
            }
        }];
    }
}

- (UIViewController *)viewControllerForRowType:(TableViewRowType)type
{
    UIViewController *viewController;
    switch (type)
    {
        case TableViewRowTypeSettings:
            viewController = [Teamchat settingsViewController];
            break;
            
        case TableViewRowTypeProfile:
            viewController = [Teamchat userProfileViewController];
            break;
            
        case TableViewRowTypePublicGroups:
        {
            __weak HomeViewController *weakself = self;
            viewController = [Teamchat publicGroupsViewController:^(TeamchatGroup *selectedGroup) {
                if (selectedGroup)
                {
                    [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
                        if (success)
                        {
                            UIViewController *chatWindowViewController = [Teamchat
                                                                          createTeamchatWindowViewControllerWithGroup:selectedGroup];
                            if (chatWindowViewController)
                            {
                                [weakself.navigationController pushViewController:chatWindowViewController animated:YES];
                            }
                        }
                    }];
                }
            }];
            break;
        }
            
        case TableViewRowTypeShowChatsList:
            viewController = [Teamchat teamchatGroupsViewController];
            break;
            
        default:
            break;
    }
    return viewController;
}

#pragma mark - UITableViewDataSource methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return ([Teamchat sessionState] == TCSessionStateActive || [Teamchat sessionState] == TCSessionStateLoggedIn) ? 10 : 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    HomeTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"HomeTableViewCell"];
    NSString *descriptionText = @"";
    NSString *imageName;
    
    switch (indexPath.row)
    {
        case TableViewRowTypeAddCustomContacts:
        {
            descriptionText = @"Add custom contacts";;
            imageName = @"contacts";
            break;
        }
            
        case TableViewRowTypeShowChatsList:
        {
            descriptionText = @"Show chats list";
            imageName = @"ChatListView";
            break;
        }
        
        case TableViewRowTypeShowChatRoom:
        {
            descriptionText = @"Show chat room";
            imageName = @"ChatWindow";
            break;
        }
            
        case TableViewRowTypeProfile:
        {
            descriptionText = @"Show user profile";
            imageName = @"profile";
            break;
        }
            
        case TableViewRowTypeSettings:
        {
            descriptionText = @"Settings";
            imageName = @"settings_icon";
            break;
        }
            
        case TableViewRowTypePublicGroups:
        {
            descriptionText = @"Show public rooms";
            imageName = @"ChatListView";
            break;
        }
            
        case TableViewRowTypeCreateNewGroup:
        {
            descriptionText = @"Create a new group";
            imageName = @"groupImage";
            break;
        }
            
        case TableViewRowTypeAddMembersToGroup:
        {
            descriptionText = @"Add members to group";
            imageName = @"contacts";
            break;
        }
        case TableViewRowTypeGroupFilter:
        {
            descriptionText = @"Group Filters";
            imageName = @"GroupFilterIcon";
            break;
        }
        case TableViewRowTypeUpdateUserProfile:
        {
            descriptionText = @"Update User Profile";
            imageName = @"updateProfile";
            break;
        }
            
        default:
            break;
    }
    
    [cell configureWithDescriptionText:descriptionText imageName:imageName];
    return cell;
}

#pragma mark - UITableViewDelegate methods

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    switch (indexPath.row)
    {
        case TableViewRowTypeAddCustomContacts:
            [self performSegueWithIdentifier:@"CustomContactsSegue" sender:nil];
            break;
            
        case TableViewRowTypeShowChatRoom:
        case TableViewRowTypeAddMembersToGroup:
            [self showGroupsList];
            break;
            
        case TableViewRowTypeShowChatsList:
        case TableViewRowTypeProfile:
        case TableViewRowTypeSettings:
        case TableViewRowTypePublicGroups:
            [self showViewForRowType:indexPath.row];
            break;
            
        case TableViewRowTypeCreateNewGroup:
            [self showCreateGroupView];
            break;
        case TableViewRowTypeGroupFilter:
            [self performSegueWithIdentifier:@"GroupFilterSeagueIdentifier" sender:nil];
            break;
        case TableViewRowTypeUpdateUserProfile:
            [self performSegueWithIdentifier:@"UpdateProfileSeagueIdentifier" sender:nil];
            break;

            
        default:
            break;
    }
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)showCreateGroupView
{
    [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
    [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
        [SVProgressHUD dismiss];
        
        if (success)
        {
            [self performSegueWithIdentifier:@"NewGroup" sender:self];
        }
    }];
}

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"SettingsSegue"])
    {
        SettingsViewController *settingsViewController = (SettingsViewController *)segue.destinationViewController;
        settingsViewController.completionHandler = ^(NSDictionary *settingsInfo)
        {
            if (settingsInfo)
            {
                if ([settingsInfo[NavigationTitleKey] isKindOfClass:[NSString class]])
                {
                    NSString *title = settingsInfo[NavigationTitleKey];
                    
                    if (title.length)
                    {
                        [Teamchat setNavigationTitle:title];
                    }
                }
                
                if ([settingsInfo[NavigationTitleColor] isKindOfClass:[UIColor class]])
                {
                    UIColor *titleColor = settingsInfo[NavigationTitleColor];
                    
                    if (titleColor)
                    {
                        [Teamchat setNavigationTitleColor:titleColor];
                    }
                }
                
                if ([settingsInfo[TableViewSeparatorColor] isKindOfClass:[UIColor class]])
                {
                    UIColor *tableViewSeparatorColor = settingsInfo[TableViewSeparatorColor];
                    
                    if (tableViewSeparatorColor)
                    {
                        [Teamchat setTableViewSeparatorColor:tableViewSeparatorColor];
                    }
                }
                if ([settingsInfo[BaseURLKey] isKindOfClass:[NSString class]])
                {
                    NSString *baseURL = settingsInfo[BaseURLKey];
                    [Teamchat setHostURL:baseURL];
                }
            }
        };
    }
    else if ([segue.identifier isEqualToString:@"LoginSegue"])
    {
        LoginViewController *loginViewController = (LoginViewController *)segue.destinationViewController;
        loginViewController.logionOptionType = self.loginOptionType;
        loginViewController.delegate = self;
    }
    else if ([segue.identifier isEqualToString:@"CustomContactsSegue"])
    {
        CustomContactsViewController *customContactViewController = (CustomContactsViewController *)segue.destinationViewController;
        [customContactViewController configureWithCustomContacts:self.customContactsArray delegate:self];
    }
    else if ([segue.identifier isEqualToString:@"RoomsListSegue"])
    {
        ListTableViewController *roomListViewController = (ListTableViewController *)segue.destinationViewController;
        TableViewRowType selectedRowType = [self.homeTableView indexPathForSelectedRow].row;
        roomListViewController.listType = selectedRowType == TableViewRowTypeAddMembersToGroup ?
                                                ListTypeToAddContactsToGroup :
                                                ListTypeToShowChatRoom;
    }
    else if ([segue.identifier isEqualToString:@"RegisterUsingVerifyCodeAPIsSegueIdentifier"])
    {
        RegisterUsingVerifyCodeAPIsViewController *registerUsingVerifyCodeAPIsViewController = (RegisterUsingVerifyCodeAPIsViewController *)segue.destinationViewController;
        registerUsingVerifyCodeAPIsViewController.flowType = TCRegistrationFlowTypeRegister;
    }
}

#pragma mark - UIActionSheetDelegate methods

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    switch (buttonIndex)
    {
        case LoginOptionTypeDefault:
        case LoginOptionTypeTeamchatLoginWithPrefilledEmail:
        {
            self.loginOptionType = buttonIndex;
            [self performSegueWithIdentifier:@"LoginSegue" sender:nil];
            break;
        }
            
        case LoginOptionTypeTeamChatLogin:
        {
            [Teamchat setUserEmailID:nil];
            [self showLoginView];
            break;
        }
            case LoginOptionTypeUsingRegisterAndVerifyCodeAPIs:
        {
            self.loginOptionType = buttonIndex;
            [self performSegueWithIdentifier:@"RegisterUsingVerifyCodeAPIsSegueIdentifier" sender:nil];
            break;
        }
            
        default:
            break;
    }
}

#pragma mark - LoginViewControllerDelegate methods

- (void)showLoginScreenForLoginOption:(LoginOptionType)loginOptionType email:(NSString *)email userID:(NSString *)userID authToken:(NSString *)authToken
{
    switch (loginOptionType)
    {
        case LoginOptionTypeTeamchatLoginWithPrefilledEmail:
        {
            if (email)
            {
                [Teamchat setUserEmailID:email];
            }
            [self showLoginView];
            break;
        }
            
        case LoginOptionTypeDefault:
        {
            [Teamchat setUserEmailID:email];
            [Teamchat setUserID:userID];
            [Teamchat setAuthenticationCode:authToken];
            
            [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
            [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
                [SVProgressHUD dismiss];
                
                if (success)
                {
                    [[[UIAlertView alloc] initWithTitle:@"Success" message:@"Teamchat login successful"
                                               delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
                    [self changeViewsOnSuccessfulLogin];
                }
                else
                {
                    NSString *errorMessage = error ? [error localizedDescription] : errMsg;
                    if (errorMessage)
                    {
                        [[[UIAlertView alloc] initWithTitle:@"Failed" message:errorMessage
                                                   delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
                    }
                }
            }];

            break;
        }
        default:
            break;
    }
}

#pragma mark - TCCustomAddressBookContactProvider methods

- (NSArray *)customAddressBookContacts
{
    return self.customContactsArray;
}

#pragma mark - CustomContactsViewControllerDelegate methods

- (void)customContacts:(NSArray *)customContacts
{
    self.customContactsArray = customContacts;
}

#pragma mark - UIAlertViewDelegate methods

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex != alertView.cancelButtonIndex)
    {
        [Teamchat resetTeamchatSession];
        [[[UIAlertView alloc] initWithTitle:@"Success" message:@"Teamchat session is reset"
                                   delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
        [self.loginOrResetBarButtonItem setTitle:@"Login"];
        
        [self.homeTableView reloadData];
    }
}

#pragma mark - TeamchatJsIOSBridgeInjectorDelegate methods

- (void)injectJsIOSBridgeFor:(UIWebView*)webView
{
    
}

- (void)handleCallbackForScheme:(NSString *)scheme URL:(NSURL *)URL
{
    [[[UIAlertView alloc] initWithTitle:@"Callback received" message:[NSString stringWithFormat:@"Callback scheme: %@", scheme] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
}

@end
