//
//  NewGroupViewController.m
//  SDKTestApp
//
//  Created by Vidyalaxmi V Shenoy on 21/10/15.
//  Copyright © 2015 Robosoft. All rights reserved.
//

#import <TeamchatSDK/TeamchatSDK.h>

#import "NewGroupViewController.h"
#import "CustomContactTableViewCell.h"
#import "NewContactViewController.h"
#import "SVProgressHUD.h"
#import "ListTableViewController.h"

@interface NewGroupViewController () <NewContactViewControllerDelegate, ListControllerDelegate, UIActionSheetDelegate>

@property (nonatomic) NSMutableArray *groupMembersArray;

@property (weak, nonatomic) IBOutlet UISwitch *adminOnlySwitch;
@property (weak, nonatomic) IBOutlet UISwitch *hideMemberProfileSwitch;
@property (weak, nonatomic) IBOutlet UITableView *membersTableView;
@property (weak, nonatomic) IBOutlet UITextField *nameTextField;

@end

@implementation NewGroupViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.membersTableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
}

#pragma mark - UITableViewDataSource methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section;
{
    return self.groupMembersArray.count + 1; // +1 to show add contact cell
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.row == 0)
    {
        UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"AddContactCell"];
        return cell;
    }
    else
    {
        CustomContactTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"MemberCell"];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        TCTeamchatContact *contact = self.groupMembersArray[indexPath.row - 1];
        [cell configureWithName:contact.profileName email:contact.email phone:contact.phone];
        return cell;
    }
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return indexPath.row == 0 ? NO : YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        [self.groupMembersArray removeObjectAtIndex:indexPath.row - 1];
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }
}

#pragma mark - UITableViewDelegate methods

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.row == 0)
    {
        return 51.f;
    }
    else
    {
        TCCustomAddressBookContact *contact = self.groupMembersArray[indexPath.row - 1];
        return [CustomContactTableViewCell heightForContact:contact];
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    if (indexPath.row == 0)
    {
        [self showContactAdditionOptions];
    }
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"NewContactSegue"])
    {
        NewContactViewController *newContactViewController = (NewContactViewController *)segue.destinationViewController;
        newContactViewController.delegate = self;
    }
}

#pragma mark - NewContactViewControllerDelegate methods

- (void)newContactViewController:(NewContactViewController *)newContactViewController addedContact:(TCTeamchatContact *)contact
{
    if (!self.groupMembersArray)
    {
        self.groupMembersArray = [NSMutableArray array];
    }
    
    NSInteger previousCount = self.groupMembersArray.count;
    [self checkForDuplicatesAndAddFromContacts:@[contact] showAlert:YES];
    if (self.groupMembersArray.count > previousCount)
    {
        [self.membersTableView insertRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:self.groupMembersArray.count inSection:0]] withRowAnimation:UITableViewRowAnimationAutomatic];
    }
}

#pragma mark - ListControllerDelegate methods

- (void)selectedContacts:(NSArray *)selectedContacts
{
    if (selectedContacts.count)
    {
        if (!self.groupMembersArray)
        {
            self.groupMembersArray = [NSMutableArray array];
        }
        
        
        [self.membersTableView beginUpdates];
        
        NSInteger startIndex = self.groupMembersArray.count;
        // Remove any duplicate contacts.
        [self checkForDuplicatesAndAddFromContacts:selectedContacts showAlert:YES];
        NSMutableArray *indexes = [NSMutableArray array];
        
        for (NSInteger i = startIndex; i < self.groupMembersArray.count; i++)
        {
            [indexes addObject:[NSIndexPath indexPathForRow:i+1 inSection:0]];
        }

        
        [self.membersTableView insertRowsAtIndexPaths:indexes withRowAnimation:UITableViewRowAnimationAutomatic];
        [self.membersTableView endUpdates];
    }
}

-(void)checkForDuplicatesAndAddFromContacts:(NSArray *)selectedContacts showAlert:(BOOL)shouldShowAlert
{
    // TODO:: Improve this method. Need to generalize.
    if (selectedContacts.count)
    {
        __block NSMutableIndexSet * filteredSet = [NSMutableIndexSet new];
        if (!self.groupMembersArray.count)
        {
            [self.groupMembersArray addObjectsFromArray:selectedContacts];
            return ;
        }
        else
        {
            [self.groupMembersArray enumerateObjectsUsingBlock:^(TCTeamchatContact *  _Nonnull contact, NSUInteger idx, BOOL * _Nonnull stop) {
                [selectedContacts enumerateObjectsUsingBlock:^(TCTeamchatContact *  _Nonnull selectedContact, NSUInteger sameContactIndex, BOOL * _Nonnull stop) {
                    if ([[selectedContact email] isEqualToString:contact.email])
                    {
                        [filteredSet addIndex:sameContactIndex];
                        *stop = YES;
                    }
                }];
            }];
            if (filteredSet.count)
            {
                NSMutableArray * filteredArray = [selectedContacts mutableCopy];
                [filteredArray removeObjectsAtIndexes:filteredSet];
                [self.groupMembersArray addObjectsFromArray:filteredArray];
                if (shouldShowAlert)
                {
                    [[[UIAlertView alloc] initWithTitle:@"Alert" message:[NSString stringWithFormat:@"Removed %ld duplicate contact.",filteredSet.count] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
                }
            }
            else
            {
                [self.groupMembersArray addObjectsFromArray:selectedContacts];
            }
        }
    }
}

#pragma mark - UIActionSheetDelegate methods

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    if (buttonIndex != actionSheet.cancelButtonIndex)
    {
        switch (buttonIndex)
        {
            case 0:
                [self showCustomContactsView];
                break;
                
            case 1:
                [self showServerContacts];
                break;
                
            default:
                break;
        }
    }
}

#pragma mark - Action methods

- (IBAction)doneButtonTapped:(id)sender
{
    if (!self.nameTextField.text.length)
    {
        [[[UIAlertView alloc] initWithTitle:@"Failed" message:@"Please type a group name"
                                   delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
    }
    else if (!self.groupMembersArray.count)
    {
        [[[UIAlertView alloc] initWithTitle:@"Failed" message:@"Please add at least one member"
                                   delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
    }
    else
    {
        // Create new group
        TeamchatGroupCreator *groupCreator = [TeamchatGroupCreator new];
        [groupCreator setGroupName:self.nameTextField.text];
        [groupCreator setGroupMembers:self.groupMembersArray];
        [groupCreator setAdminOnly:self.adminOnlySwitch.isOn];
        [groupCreator setShouldHideMemberProfiles:self.hideMemberProfileSwitch.isOn];
        
        [SVProgressHUD showWithMaskType:SVProgressHUDMaskTypeBlack];
        [groupCreator createGroupWithCompletionHandler:^(BOOL success, NSError *error, NSString *msg, TeamchatGroup *createdGroup) {
            
            [SVProgressHUD dismiss];
            
            NSString *title = success ? @"Success" : @"Failed";
            NSString *alertMessage = success ? @"Group creation successful" : error ? [error localizedDescription] : msg ? : nil;
            [[[UIAlertView alloc] initWithTitle:title message:alertMessage
                                       delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
            
            [self.navigationController popViewControllerAnimated:YES];
        }];
    }
}

- (void)showContactAdditionOptions
{
    if (self.nameTextField.isFirstResponder)
    {
        [self.nameTextField resignFirstResponder];
    }
    UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:nil delegate:self cancelButtonTitle:@"Cancel" destructiveButtonTitle:nil otherButtonTitles:@"Add custom contacts",@"Add Teamchat contacts", nil];
    [actionSheet showInView:self.view];
}

- (void)showCustomContactsView
{
    [self performSegueWithIdentifier:@"NewContactSegue" sender:self];
}

- (void)showServerContacts
{
    ListTableViewController *serverContactsViewController = [self.storyboard instantiateViewControllerWithIdentifier:@"RoomsList"];
    serverContactsViewController.listType = ListTypeServerContacts;
    serverContactsViewController.delegate = self;
    
    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:serverContactsViewController];
    [self presentViewController:navController animated:YES completion:nil];
}

@end
