//
//  RegisterUsingVerifyCodeAPIsViewController.m
//  SDKTestApp
//
//  Created by Vishwesh Joshi on 09/12/15.
//  Copyright © 2015 Robosoft. All rights reserved.
//

#import "RegisterUsingVerifyCodeAPIsViewController.h"
#import <TeamchatSDK/TeamchatSDK.h>
#import "SVProgressHUD.h"

@interface RegisterUsingVerifyCodeAPIsViewController ()
@property (weak, nonatomic) IBOutlet UITextField *emailTextField;
@property (weak, nonatomic) IBOutlet UIButton *sendVerificationOrVerifyCodeButton;
@property (weak, nonatomic) IBOutlet UILabel *emailOrVerifyCodeTitleLabel;

@end

@implementation RegisterUsingVerifyCodeAPIsViewController

+ (instancetype)instance
{
   return [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"RegisterUsingVerifyCodeAPIsViewControllerIdentifier"];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self configureButtonsAndLabels];
}

- (IBAction)handleSendVerificationCodeOrVerifyCode:(UIButton *)sender
{
    switch (self.flowType)
    {
        case TCRegistrationFlowTypeRegister:
        {
            [self registerWithEmail:self.emailTextField.text];
            break;
        }
        case TCRegistrationFlowTypeVerifyCode:
        {
            [self verifyCodeForRegisteredEmail:self.emailTextField.text];
            break;
        }
            
        default:
            break;
    }
}

- (void)configureButtonsAndLabels
{
    self.emailTextField.text = nil;
    switch (self.flowType)
    {
        case TCRegistrationFlowTypeRegister:
        {
            [self changeLabelTitle:NSLocalizedString(@"Enter email ID", "Email ID title")
                    andButtonTitle:NSLocalizedString(@"Send Verification Code", "Send Verification Code title")];
            break;
        }
        case TCRegistrationFlowTypeVerifyCode:
        {
            [self changeLabelTitle:NSLocalizedString(@"Enter Verification Code", "Email ID title")
                    andButtonTitle:NSLocalizedString(@"Verify Code", "Send Verification Code title")];
            break;
        }
            
        default:
            break;
    }
}

- (void)changeLabelTitle:(NSString *)title andButtonTitle:(NSString *)buttonTitle
{
    self.emailOrVerifyCodeTitleLabel.text = title;
    [self.sendVerificationOrVerifyCodeButton setTitle:buttonTitle forState:UIControlStateNormal];
}

-(void)registerWithEmail:(NSString *)emailID
{
    [SVProgressHUD show];
    [self enableOrDisableViewInteraction:NO];
    [Teamchat registerUserWithEmail:emailID withCompletion:^(BOOL success, NSError *error, NSString *errMsg) {
        [SVProgressHUD dismiss];
        [self enableOrDisableViewInteraction:YES];
        if (success)
        {
           RegisterUsingVerifyCodeAPIsViewController * verifyCodeController = [RegisterUsingVerifyCodeAPIsViewController instance];
            verifyCodeController.flowType = TCRegistrationFlowTypeVerifyCode;
            [self.navigationController pushViewController:verifyCodeController animated:NO];
        }
        else
        {
            [[[UIAlertView alloc] initWithTitle:@"Failed" message:(errMsg ? :(error.localizedDescription ? : @"Unable to register with this email")) delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
        }
    }];
}


-(void)verifyCodeForRegisteredEmail:(NSString *)verificationCode
{
    [SVProgressHUD show];
    [self enableOrDisableViewInteraction:NO];
    [Teamchat verifyRegisteredEmailWithCode:verificationCode withCompletion:^(BOOL success, NSError *error, NSString *errMsg) {
        [SVProgressHUD dismiss];
        [self enableOrDisableViewInteraction:YES];
        if (success)
        {
            [[[UIAlertView alloc] initWithTitle:@"Success" message:@"Logged in successfully" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
            [self.navigationController popToRootViewControllerAnimated:YES];
            
        }
        else
        {
            [[[UIAlertView alloc] initWithTitle:@"Failed" message:(errMsg ? : (error ? error.localizedDescription : @"Login Failed. Please try again")) delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
            [self dismissViewControllerAnimated:YES completion:nil];
        }
    }];

}

- (void)enableOrDisableViewInteraction:(BOOL)isUserInteractionEnabled
{
    self.view.userInteractionEnabled = isUserInteractionEnabled;
    self.navigationController.view.userInteractionEnabled = isUserInteractionEnabled;
}

@end
