//
//  SettingsViewController.m
//  SDKTestApp
//
//  Created by Vidyalaxmi V Shenoy on 30/07/15.
//  Copyright (c) 2015 Robosoft. All rights reserved.
//

#import "SettingsViewController.h"
#import <TeamchatSDK/TeamchatSDK.h>

NSString* NavigationTitleKey = @"NavigationTitle";
NSString* NavigationTitleColor = @"NavigationTitleColor";
NSString* TableViewSeparatorColor = @"TableViewSeparatorColor";
NSString *BaseURLKey = @"BaseURL";

static NSString *kRedColor = @"Red";
static NSString *kBlueColor = @"Blue";
static NSString *kGreenColor = @"Green";
static NSString *kYellowColor = @"Yellow";

NSInteger const kSandBoxingAlertTag = 100;

@interface SettingsViewController () <UIPickerViewDataSource, UIPickerViewDelegate,UIAlertViewDelegate>

@property (weak, nonatomic) IBOutlet UITextField *navigationTitleTextField;
@property (weak, nonatomic) IBOutlet UITextField *navigationTitleColorTextField;
@property (weak, nonatomic) IBOutlet UITextField *tableViewSeparatorColorTextField;
@property (weak, nonatomic) IBOutlet UITextField *baseURLTextField;
@property (weak, nonatomic) IBOutlet UITextField *pnEnvironmentTextField;

@property (nonatomic) UITextField *activeTextField;
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UISwitch *sandboxingSwitch;

@property (nonatomic, copy) UIColor *selectedTitleColor;
@property (nonatomic, copy) UIColor *selectedSeparatorColor;
@property (nonatomic, copy) NSString *selectedBaseURL;

@property (nonatomic) NSArray *colors;

@end

@implementation SettingsViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.colors = @[kRedColor, kBlueColor, kGreenColor, kYellowColor];
    [self configurePickerViewForTextField:self.navigationTitleColorTextField];
    [self configurePickerViewForTextField:self.tableViewSeparatorColorTextField];
    [self prefillTextFields];
    [self registerToKeyboardNotifications];
    
    if ([Teamchat sessionState] > TCSessionStateInactive)
    {
        self.baseURLTextField.enabled = self.pnEnvironmentTextField.enabled = NO;
    }
    if ([Teamchat sessionState] != TCSessionStateActive)
    {
        self.sandboxingSwitch.enabled = NO;
    }
    else
    {
        self.sandboxingSwitch.enabled = YES;
        self.sandboxingSwitch.on = [Teamchat isSandBoxingEnabled];
    }
}

- (void)configurePickerViewForTextField:(UITextField *)textField
{
    UIPickerView *pickerView = [UIPickerView new];
    pickerView.delegate = self;
    pickerView.dataSource = self;
    textField.inputView = pickerView;
    
    [self addInputAccessoryViewForTextField:textField];
}

- (void)prefillTextFields
{
    if ([Teamchat navigationTitle])
    {
        self.navigationTitleTextField.text = [Teamchat navigationTitle];
    }
    if ([Teamchat navigationTitleColor])
    {
        self.navigationTitleColorTextField.text = [self colorStringFromColor:[Teamchat navigationTitleColor]];
    }
    if ([Teamchat tableViewSeparatorColor])
    {
        self.tableViewSeparatorColorTextField.text = [self colorStringFromColor:[Teamchat tableViewSeparatorColor]];
    }
    
    if ([Teamchat currentHostURL].length)
    {
        self.baseURLTextField.text = [Teamchat currentHostURL];
    }
    else
    {
        // Default is PROD_APPSTORE
        self.baseURLTextField.text = @"https://enterprise.teamchat.com/";
    }
}

- (NSString *)colorStringFromColor:(UIColor *)color
{
    NSString *colorString;
    if ([color isEqual:[UIColor redColor]])
    {
        colorString = kRedColor;
    }
    else if ([color isEqual:[UIColor blueColor]])
    {
        colorString = kBlueColor;
    }
    else if ([color isEqual:[UIColor greenColor]])
    {
        colorString = kGreenColor;
    }
    else if ([color isEqual:[UIColor yellowColor]])
    {
        colorString = kYellowColor;
    }
    return colorString;
}

- (void)addInputAccessoryViewForTextField:(UITextField *)textField
{
    CGRect accessFrame = CGRectMake(0.0, 0.0, 768.0, 44.0);
    UIView *inputAccessoryView = [[UIView alloc] initWithFrame:accessFrame];
    
    UIButton *doneButton = [[UIButton alloc] initWithFrame:CGRectMake([UIScreen mainScreen].bounds.size.width - 75, 0.0, 90, 44.0)];
    [doneButton setTitle:@"Set"
                forState:UIControlStateNormal];
    
    [doneButton addTarget:self
                   action:@selector(setButtonTapped:)
         forControlEvents:UIControlEventTouchUpInside];
    
    [inputAccessoryView addSubview:doneButton];
    inputAccessoryView.backgroundColor = [UIColor colorWithRed:156.f/255.f green:163.f/255.f blue:177.f/255.f alpha:1];
    textField.inputAccessoryView = inputAccessoryView;
}

- (void)registerToKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardDidShow:)
                                                 name:UIKeyboardDidShowNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillBeHidden:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}

#pragma mark - Action methods

- (void)keyboardDidShow:(NSNotification *)notification
{
    NSDictionary* info = [notification userInfo];
    CGRect kbRect = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue];
    kbRect = [self.view convertRect:kbRect fromView:nil];
    
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, kbRect.size.height, 0.0);
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
    
    CGRect aRect = self.view.frame;
    aRect.size.height -= kbRect.size.height;
    if (!CGRectContainsPoint(aRect, self.activeTextField.frame.origin) ) {
        [self.scrollView scrollRectToVisible:self.activeTextField.frame animated:YES];
    }
}

- (void)keyboardWillBeHidden:(NSNotification *)notification
{
    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    self.scrollView.contentInset = contentInsets;
    self.scrollView.scrollIndicatorInsets = contentInsets;
}

- (void)setButtonTapped:(id)sender
{
    [self.view endEditing:YES];
}

- (IBAction)viewTapHandler:(id)sender
{
    [self.view endEditing:YES];
}

- (IBAction)dismissSettingsView:(id)sender
{
    [self.view endEditing:YES];
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)doneButtonTapped:(id)sender
{
    [self.view endEditing:YES];
    NSDictionary *settingsInfo = @{NavigationTitleColor : self.selectedTitleColor ? : @"",
                                   NavigationTitleKey : self.navigationTitleTextField.text ? : @"",
                                   TableViewSeparatorColor : self.selectedSeparatorColor ? : @"",
                                   BaseURLKey : self.selectedBaseURL ? : @""};
    if (self.completionHandler)
    {
        self.completionHandler(settingsInfo);
    }
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - UITextFieldDelegate methods

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    self.activeTextField = textField;
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    if ([textField isEqual:self.navigationTitleColorTextField] || [textField isEqual:self.tableViewSeparatorColorTextField])
    {
        return NO;
    }
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    if ([self.activeTextField isEqual:self.navigationTitleColorTextField])
    {
        if (!self.selectedTitleColor)
        {
            [self.navigationTitleColorTextField setText:self.colors.firstObject];
            self.selectedTitleColor = [UIColor redColor];
        }
    }
    else if ([self.activeTextField isEqual:self.tableViewSeparatorColorTextField])
    {
        if (!self.selectedSeparatorColor)
        {
            [self.tableViewSeparatorColorTextField setText:self.colors.firstObject];
            self.selectedSeparatorColor = [UIColor redColor];
        }
    }
    else if ([self.activeTextField isEqual:self.baseURLTextField])
    {
        self.selectedBaseURL = self.baseURLTextField.text;
    }
    else if ([self.activeTextField isEqual:self.pnEnvironmentTextField])
    {
        if (self.pnEnvironmentTextField.text.length)
        {
            [Teamchat setPNEnvironmentID:self.pnEnvironmentTextField.text];
        }
    }
    
    self.activeTextField = nil;
}

#pragma mark - UIPickerViewDataSource methods

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return self.colors.count;
}

#pragma mark - UIPickerViewDelegate methods

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    return self.colors[row];
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    self.activeTextField.text = self.colors[row];
    
    UIColor *titleColor;
    switch (row)
    {
        case 0:
            titleColor = [UIColor redColor];
            break;
            
        case 1:
            titleColor = [UIColor blueColor];
            break;
            
        case 2:
            titleColor = [UIColor greenColor];
            break;
            
        case 3:
            titleColor = [UIColor yellowColor];
            break;
            
        default:
            break;
    }
    
    if ([self.activeTextField isEqual:self.navigationTitleColorTextField])
    {
        self.selectedTitleColor = titleColor;
    }
    else
    {
        self.selectedSeparatorColor = titleColor;
    }
}

-(UIView *)pickerView:(UIPickerView *)pickerView viewForRow:(NSInteger)row forComponent:(NSInteger)component reusingView:(UIView *)view
{
    UILabel *pickerLabel = (UILabel *)view;
    
    if (!pickerLabel)
    {
        pickerLabel = [[UILabel alloc] init];
        pickerLabel.backgroundColor = [UIColor clearColor];
        pickerLabel.textAlignment = NSTextAlignmentCenter;
    }
    
    pickerLabel.textAlignment = NSTextAlignmentCenter;
    pickerLabel.text = self.colors[row];
    
    return pickerLabel;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - Sandboxing Feature

- (IBAction)enableOrDisableSandBoxing:(UISwitch *)sender {
    
    NSString *title = @"Alert", *message;
    if (sender.isOn)
    {
        message = @"Enabling Sandboxing saves downloaded images/videos securely to App directory and not gallery. Do you want to continue?";
    }
    else
    {
        message = @" Disabling sandboxing saves downloded images/videos to Gallery. Do you want to continue?";
    }

    UIAlertView * sandboxingAlert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:@"NO" otherButtonTitles:@"Continue", nil];
    sandboxingAlert.tag = kSandBoxingAlertTag;
    [sandboxingAlert show];
}

#pragma mark - UIAlertView Delegate

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    if (buttonIndex != alertView.cancelButtonIndex)
    {
        [Teamchat setSandBoxingEnabled:self.sandboxingSwitch.isOn];
    }
    else
    {
        self.sandboxingSwitch.on = !self.sandboxingSwitch.isOn;
    }
}

@end
