//
//  UpdateProfileViewController.m
//  SDKTestApp
//
//  Created by Vishwesh Joshi on 21/12/15.
//  Copyright © 2015 Robosoft. All rights reserved.
//

#import "UpdateProfileViewController.h"
#import <TeamchatSDK/TeamchatSDK.h>
#import "SVProgressHUD.h"

@interface UpdateProfileViewController () <UICollectionViewDataSource,UICollectionViewDelegate,UICollectionViewDelegateFlowLayout,UITextFieldDelegate>
@property (weak, nonatomic) IBOutlet UICollectionView *imageCollectionView;
@property (weak, nonatomic) IBOutlet UITextField *profileNametextField;
@property (weak, nonatomic) IBOutlet UITextField *countryCodeTextField;
@property (weak, nonatomic) IBOutlet UITextField *phoneNumberTextField;
@property (weak, nonatomic) IBOutlet UISwitch *updateExcludingProfileImageSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *removeProfileImageSwitch;
@property (nonatomic) NSIndexPath *previousIndexPath;

@property (nonatomic) NSArray * imagesArray;

@end

@implementation UpdateProfileViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.imagesArray = @[@"profileTest1",@"profileTest2",@"profileTest3"];
    // Do any additional setup after loading the view.
}

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self showOrHideActivityIndicator:YES];
    [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
        if (success)
        {
            [self showOrHideActivityIndicator:NO];
            [self showCurrentProfileInfo];
        }
        else
        {
            [SVProgressHUD dismiss];
            [[[UIAlertView alloc] initWithTitle:@"Failed!" message:@"Failed to initialize" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
            self.view.userInteractionEnabled = NO;
        }
    }];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    self.previousIndexPath = [NSIndexPath indexPathForItem:0 inSection:0];
    [self collectionView:self.imageCollectionView didSelectItemAtIndexPath:self.previousIndexPath];
}


- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    return self.imagesArray.count;
}

// The cell that is returned must be retrieved from a call to -dequeueReusableCellWithReuseIdentifier:forIndexPath:
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    UICollectionViewCell * cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"ProfileImageCellIdentifier" forIndexPath:indexPath];
    
    cell.backgroundView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:self.imagesArray[indexPath.row]]];
    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
    return CGSizeMake((collectionView.frame.size.width/3) - 10 , (collectionView.frame.size.height));
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout insetForSectionAtIndex:(NSInteger)section
{
   return UIEdgeInsetsMake(0, 10, 0, 10);
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    [[self activeTextField] resignFirstResponder];
    UICollectionViewCell *cell;
    if (self.previousIndexPath)
    {
        cell = [collectionView cellForItemAtIndexPath:self.previousIndexPath];
        cell.layer.borderColor = [UIColor clearColor].CGColor;
    }
    cell = [collectionView cellForItemAtIndexPath:indexPath];
    cell.layer.borderColor = [UIColor redColor].CGColor;
    cell.layer.borderWidth = 2.0f;
    self.previousIndexPath = indexPath;
    
}

- (IBAction)updateProfileTapped:(UIButton *)sender
{
    if (![self validateAllFields])
    {
        [[[UIAlertView alloc] initWithTitle:@"Failed" message:@"Check if\n 1.Name is empty\n2. Country code is added \n 3. Phone number is invalid" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
        return;
    }
    
    [self showOrHideActivityIndicator:YES];
    [Teamchat updateProfileWithName:self.profileNametextField.text.length ? self.profileNametextField.text: [TeamchatUser profileName]
                              image:(self.removeProfileImageSwitch.isOn)? nil: [UIImage imageNamed:self.imagesArray[self.previousIndexPath.row]]
                        phoneNumber:self.phoneNumberTextField.text
                        countryCode:(self.phoneNumberTextField.text.length) ? self.countryCodeTextField.text : nil
                   userCurrentImage:self.updateExcludingProfileImageSwitch.isOn
              withCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
                  [self showOrHideActivityIndicator:NO];
                  if (success)
                  {
                      [[[UIAlertView alloc] initWithTitle:@"Success" message:@"Profile Updated Successfully" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
                      [self clearAlltextFieldsAndReset];
                      [self showCurrentProfileInfo];
                  }
                  else
                  {
                      [[[UIAlertView alloc] initWithTitle:@"Failed" message:(errMsg ? :(error.localizedDescription ? : @"Failed Updating Profile")) delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
                  }
              }];
}

- (IBAction)showUserProfileTapped:(UIButton *)sender
{
    [Teamchat initWithCompletionHandler:^(BOOL success, NSError *error, NSString *errMsg) {
        if (success)
        {
            UIViewController *userProfileController = [Teamchat userProfileViewController];
            [self.navigationController pushViewController:userProfileController animated:YES];
        }
    }];
}
- (IBAction)removeProfileSwitchStateChanged:(UISwitch *)sender
{
    if (sender.isOn)
    {
        self.updateExcludingProfileImageSwitch.on = NO;
    }
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    return YES;
}

-(void)clearAlltextFieldsAndReset
{
    self.profileNametextField.text = nil;
    self.countryCodeTextField.text = nil;
    self.phoneNumberTextField.text = nil;
    self.updateExcludingProfileImageSwitch.on = YES;
    self.removeProfileImageSwitch.on = NO;
    self.previousIndexPath = [NSIndexPath indexPathForItem:0 inSection:0];
    [self.imageCollectionView reloadData];
}


- (BOOL)validateAllFields
{
    BOOL isValidInput = YES;
    if (!self.profileNametextField.text.length)
    {
        isValidInput = NO;
    }
    if (self.phoneNumberTextField.text.length && !self.countryCodeTextField.text.length)
    {
        isValidInput = NO;
    }
    return isValidInput;
}

-(void)showCurrentProfileInfo
{
    self.profileNametextField.text = self.profileNametextField.placeholder = [TeamchatUser profileName];
    self.countryCodeTextField.text = [TeamchatUser phoneNumber].length ? [NSString stringWithFormat:@"+%@",[TeamchatUser countryCode]] : nil;
    self.phoneNumberTextField.text = self.phoneNumberTextField.placeholder = [TeamchatUser phoneNumber];
    [[self activeTextField] resignFirstResponder]
    ;
}

- (UITextField *)activeTextField
{
    UITextField *activeTextField = (self.profileNametextField.isFirstResponder) ? self.profileNametextField : (self.countryCodeTextField.isFirstResponder) ? self.countryCodeTextField : self.phoneNumberTextField;
    return activeTextField;
}

-(void)showOrHideActivityIndicator:(BOOL)shouldShow
{
    if (shouldShow)
    {
        [SVProgressHUD show];
        self.view.userInteractionEnabled = NO;
    }
    else
    {
        [SVProgressHUD dismiss];
        self.view.userInteractionEnabled = YES;
    }
}

-(void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [super touchesBegan:touches withEvent:event];
    [[self activeTextField] resignFirstResponder];
}

@end
